package sun.security.pkcs12;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateFactory;
import java.security.InvalidAlgorithmParameterException;
import java.io.InputStream;
import java.util.Iterator;
import javax.crypto.Mac;
import java.security.MessageDigest;
import java.security.PKCS12Attribute;
import java.security.UnrecoverableEntryException;
import java.security.cert.CertificateException;
import sun.security.pkcs.ContentInfo;
import java.io.OutputStream;
import java.util.Enumeration;
import java.security.spec.AlgorithmParameterSpec;
import java.security.AccessController;
import java.security.Security;
import java.security.PrivilegedAction;
import java.io.UnsupportedEncodingException;
import java.util.HashSet;
import sun.security.util.DerOutputStream;
import java.util.Set;
import java.security.KeyStoreException;
import javax.security.auth.DestroyFailedException;
import java.security.KeyStore;
import java.util.Date;
import java.util.Arrays;
import java.security.cert.Certificate;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import javax.crypto.SecretKey;
import java.security.AlgorithmParameters;
import sun.security.util.DerInputStream;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.KeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.SecretKeySpec;
import sun.security.x509.AlgorithmId;
import javax.crypto.Cipher;
import java.security.spec.InvalidParameterSpecException;
import javax.crypto.spec.PBEParameterSpec;
import java.io.IOException;
import sun.security.util.DerValue;
import sun.security.pkcs.EncryptedPrivateKeyInfo;
import java.security.UnrecoverableKeyException;
import java.util.Locale;
import java.security.Key;
import java.util.Collections;
import java.security.cert.X509Certificate;
import javax.security.auth.x500.X500Principal;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.Map;
import java.security.SecureRandom;
import sun.security.util.ObjectIdentifier;
import sun.security.util.Debug;
import java.security.KeyStoreSpi;

public final class PKCS12KeyStore extends KeyStoreSpi
{
    public static final int VERSION_3 = 3;
    private static final String[] KEY_PROTECTION_ALGORITHM;
    private static final int MAX_ITERATION_COUNT = 5000000;
    private static final int PBE_ITERATION_COUNT = 50000;
    private static final int MAC_ITERATION_COUNT = 100000;
    private static final int SALT_LEN = 20;
    private static final String[] CORE_ATTRIBUTES;
    private static final Debug debug;
    private static final int[] keyBag;
    private static final int[] certBag;
    private static final int[] secretBag;
    private static final int[] pkcs9Name;
    private static final int[] pkcs9KeyId;
    private static final int[] pkcs9certType;
    private static final int[] pbeWithSHAAnd40BitRC2CBC;
    private static final int[] pbeWithSHAAnd3KeyTripleDESCBC;
    private static final int[] pbes2;
    private static final int[] TrustedKeyUsage;
    private static final int[] AnyExtendedKeyUsage;
    private static ObjectIdentifier PKCS8ShroudedKeyBag_OID;
    private static ObjectIdentifier CertBag_OID;
    private static ObjectIdentifier SecretBag_OID;
    private static ObjectIdentifier PKCS9FriendlyName_OID;
    private static ObjectIdentifier PKCS9LocalKeyId_OID;
    private static ObjectIdentifier PKCS9CertType_OID;
    private static ObjectIdentifier pbeWithSHAAnd40BitRC2CBC_OID;
    private static ObjectIdentifier pbeWithSHAAnd3KeyTripleDESCBC_OID;
    private static ObjectIdentifier pbes2_OID;
    private static ObjectIdentifier TrustedKeyUsage_OID;
    private static ObjectIdentifier[] AnyUsage;
    private int counter;
    private int privateKeyCount;
    private int secretKeyCount;
    private int certificateCount;
    private SecureRandom random;
    private Map<String, PKCS12KeyStore.Entry> entries;
    private ArrayList<PKCS12KeyStore.KeyEntry> keyList;
    private LinkedHashMap<X500Principal, X509Certificate> certsMap;
    private ArrayList<PKCS12KeyStore.CertEntry> certEntries;
    
    public PKCS12KeyStore() {
        this.counter = 0;
        this.privateKeyCount = 0;
        this.secretKeyCount = 0;
        this.certificateCount = 0;
        this.entries = Collections.synchronizedMap(new LinkedHashMap());
        this.keyList = new ArrayList();
        this.certsMap = new LinkedHashMap();
        this.certEntries = new ArrayList();
    }
    
    @Override
    public Key engineGetKey(String s, char[] array) throws NoSuchAlgorithmException, UnrecoverableKeyException {
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(s.toLowerCase(Locale.ENGLISH));
        if (entry == null || !(entry instanceof PKCS12KeyStore.KeyEntry)) {
            return null;
        }
        byte[] encoded;
        if ((entry instanceof PKCS12KeyStore.PrivateKeyEntry)) {
            encoded = ((PKCS12KeyStore.PrivateKeyEntry)entry).protectedPrivKey;
        }
        else if ((entry instanceof PKCS12KeyStore.SecretKeyEntry)) {
            encoded = ((PKCS12KeyStore.SecretKeyEntry)entry).protectedSecretKey;
        }
        else {
            throw new UnrecoverableKeyException("Error locating key");
        }
        byte[] encryptedData;
        ObjectIdentifier oid;
        AlgorithmParameters algParameters;
        try {
            EncryptedPrivateKeyInfo o = new EncryptedPrivateKeyInfo(encoded);
            encryptedData = o.getEncryptedData();
            DerInputStream derInputStream = new DerValue(o.getAlgorithm().encode()).toDerInputStream();
            oid = derInputStream.getOID();
            algParameters = this.parseAlgParameters(oid, derInputStream);
        }
        catch (IOException ex) {
            UnrecoverableKeyException o2 = new UnrecoverableKeyException("Private key not stored as PKCS#8 EncryptedPrivateKeyInfo: " + ex);
            o2.initCause(ex);
            throw o2;
        }
        PrivateKey privateKey;
        try {
            int iterationCount = 0;
            if (algParameters != null) {
                PBEParameterSpec pbeParameterSpec;
                try {
                    pbeParameterSpec = (PBEParameterSpec)algParameters.getParameterSpec(PBEParameterSpec.class);
                }
                catch (InvalidParameterSpecException ex2) {
                    throw new IOException("Invalid PBE algorithm parameters");
                }
                iterationCount = pbeParameterSpec.getIterationCount();
                if (iterationCount > 5000000) {
                    throw new IOException("PBE iteration count too large");
                }
            }
            byte[] doFinal;
            Label_0250: {
                try {
                    SecretKey pbeKey = this.getPBEKey(array);
                    Cipher instance = Cipher.getInstance(mapPBEParamsToAlgorithm(oid, algParameters));
                    instance.init(2, pbeKey, algParameters);
                    doFinal = instance.doFinal(encryptedData);
                }
                catch (Exception ex3) {
                    if (array.length == 0) {
                        array = new char[] { '\0' };
                        continue;
                    }
                    throw ex3;
                }
            }
            DerInputStream derInputStream2 = new DerValue(doFinal).toDerInputStream();
            derInputStream2.getInteger();
            DerValue[] sequence = derInputStream2.getSequence(2);
            if (sequence.length < 1 || sequence.length > 2) {
                throw new IOException("Invalid length for AlgorithmIdentifier");
            }
            String name = new AlgorithmId(sequence[0].getOID()).getName();
            if ((entry instanceof PKCS12KeyStore.PrivateKeyEntry)) {
                privateKey = KeyFactory.getInstance(name).generatePrivate(new PKCS8EncodedKeySpec(doFinal));
                if (PKCS12KeyStore.debug != null) {
                    PKCS12KeyStore.debug.println("Retrieved a protected private key at alias '" + s + "' (" + new AlgorithmId(oid).getName() + " iterations: " + iterationCount + ")");
                }
            }
            else {
                SecretKeySpec key = new SecretKeySpec(derInputStream2.getOctetString(), name);
                if (name.startsWith("PBE")) {
                    SecretKeyFactory instance2 = SecretKeyFactory.getInstance(name);
                    privateKey = instance2.generateSecret(instance2.getKeySpec(key, PBEKeySpec.class));
                }
                else {
                    privateKey = key;
                }
                if (PKCS12KeyStore.debug != null) {
                    PKCS12KeyStore.debug.println("Retrieved a protected secret key at alias '" + s + "' (" + new AlgorithmId(oid).getName() + " iterations: " + iterationCount + ")");
                }
            }
        }
        catch (Exception cause) {
            UnrecoverableKeyException o3 = new UnrecoverableKeyException("Get Key failed: " + cause.getMessage());
            o3.initCause(cause);
            throw o3;
        }
        return privateKey;
    }
    
    @Override
    public Certificate[] engineGetCertificateChain(String str) {
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(str.toLowerCase(Locale.ENGLISH));
        if (entry != null && entry instanceof PKCS12KeyStore.PrivateKeyEntry) {
            if (((PKCS12KeyStore.PrivateKeyEntry)entry).chain == null) {
                return null;
            }
            if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("Retrieved a " + ((PKCS12KeyStore.PrivateKeyEntry)entry).chain.length + "-certificate chain at alias '" + str + "'");
            }
            return (Certificate[])((PKCS12KeyStore.PrivateKeyEntry)entry).chain.clone();
        }
        else {
            return null;
        }
    }
    
    @Override
    public Certificate engineGetCertificate(String str) {
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(str.toLowerCase(Locale.ENGLISH));
        if (entry == null) {
            return null;
        }
        if (entry instanceof PKCS12KeyStore.CertEntry && ((PKCS12KeyStore.CertEntry)entry).trustedKeyUsage != null) {
            if (PKCS12KeyStore.debug != null) {
                if (Arrays.equals(PKCS12KeyStore.AnyUsage, ((PKCS12KeyStore.CertEntry)entry).trustedKeyUsage)) {
                    PKCS12KeyStore.debug.println("Retrieved a certificate at alias '" + str + "' (trusted for any purpose)");
                }
                else {
                    PKCS12KeyStore.debug.println("Retrieved a certificate at alias '" + str + "' (trusted for limited purposes)");
                }
            }
            return ((PKCS12KeyStore.CertEntry)entry).cert;
        }
        else if ((entry instanceof PKCS12KeyStore.PrivateKeyEntry)) {
            if (((PKCS12KeyStore.PrivateKeyEntry)entry).chain == null) {
                return null;
            }
            if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("Retrieved a certificate at alias '" + str + "'");
            }
            return ((PKCS12KeyStore.PrivateKeyEntry)entry).chain[0];
        }
        else {
            return null;
        }
    }
    
    @Override
    public Date engineGetCreationDate(String s) {
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(s.toLowerCase(Locale.ENGLISH));
        if (entry != null) {
            return new Date(entry.date.getTime());
        }
        return null;
    }
    
    @Override
    public synchronized void engineSetKeyEntry(String s, Key key, char[] password, Certificate[] array) throws KeyStoreException {
        KeyStore.PasswordProtection passwordProtection = new KeyStore.PasswordProtection(password);
        try {
            this.setKeyEntry(s, key, passwordProtection, array, null);
        }
        finally {
            try {
                passwordProtection.destroy();
            }
            catch (DestroyFailedException ex) {}
        }
    }
    
    private void setKeyEntry(String str, Key key, KeyStore.PasswordProtection passwordProtection, Certificate[] array, Set<KeyStore.Entry.Attribute> set) throws KeyStoreException {
        try {
            PKCS12KeyStore.PrivateKeyEntry privateKeyEntry;
            if ((key instanceof PrivateKey)) {
                privateKeyEntry = new PKCS12KeyStore.PrivateKeyEntry();
                privateKeyEntry.date = new Date();
                if (key.getFormat().equals("PKCS#8") || key.getFormat().equals("PKCS8")) {
                    if (PKCS12KeyStore.debug != null) {
                        PKCS12KeyStore.debug.println("Setting a protected private key at alias '" + str + "'");
                    }
                    privateKeyEntry.protectedPrivKey = this.encryptPrivateKey(key.getEncoded(), passwordProtection);
                    if (array != null) {
                        if (array.length > 1 && !this.validateChain(array)) {
                            throw new KeyStoreException("Certificate chain is not valid");
                        }
                        privateKeyEntry.chain = (Certificate[])array.clone();
                        this.certificateCount += array.length;
                        if (PKCS12KeyStore.debug != null) {
                            PKCS12KeyStore.debug.println("Setting a " + array.length + "-certificate chain at alias '" + str + "'");
                        }
                    }
                    ++this.privateKeyCount;
                }
                else {
                    throw new KeyStoreException("Private key is not encodedas PKCS#8");
                }
            }
            else if ((key instanceof SecretKey)) {
                PKCS12KeyStore.SecretKeyEntry secretKeyEntry = new PKCS12KeyStore.SecretKeyEntry();
                secretKeyEntry.date = new Date();
                DerOutputStream derOutputStream = new DerOutputStream();
                DerOutputStream out = new DerOutputStream();
                out.putInteger(0);
                AlgorithmId.get(key.getAlgorithm()).encode(out);
                out.putOctetString(key.getEncoded());
                derOutputStream.write((byte)48, out);
                secretKeyEntry.protectedSecretKey = this.encryptPrivateKey(derOutputStream.toByteArray(), passwordProtection);
                if (PKCS12KeyStore.debug != null) {
                    PKCS12KeyStore.debug.println("Setting a protected secret key at alias '" + str + "'");
                }
                ++this.secretKeyCount;
                privateKeyEntry = secretKeyEntry;
            }
            else {
                throw new KeyStoreException("Unsupported Key type");
            }
            privateKeyEntry.attributes = new HashSet();
            if (set != null) {
                privateKeyEntry.attributes.addAll(set);
            }
            privateKeyEntry.keyId = ("Time " + privateKeyEntry.date.getTime()).getBytes("UTF8");
            privateKeyEntry.alias = str.toLowerCase(Locale.ENGLISH);
            this.entries.put(str.toLowerCase(Locale.ENGLISH), privateKeyEntry);
        }
        catch (Exception ex) {
            throw new KeyStoreException("Key protection  algorithm not found: " + ex, ex);
        }
    }
    
    @Override
    public synchronized void engineSetKeyEntry(String s, byte[] encoded, Certificate[] array) throws KeyStoreException {
        try {
            new EncryptedPrivateKeyInfo(encoded);
        }
        catch (IOException ex) {
            throw new KeyStoreException("Private key is not stored as PKCS#8 EncryptedPrivateKeyInfo: " + ex, ex);
        }
        PKCS12KeyStore.PrivateKeyEntry privateKeyEntry = new PKCS12KeyStore.PrivateKeyEntry();
        privateKeyEntry.date = new Date();
        if (PKCS12KeyStore.debug != null) {
            PKCS12KeyStore.debug.println("Setting a protected private key at alias '" + s + "'");
        }
        try {
            privateKeyEntry.keyId = ("Time " + privateKeyEntry.date.getTime()).getBytes("UTF8");
        }
        catch (UnsupportedEncodingException ex2) {}
        privateKeyEntry.alias = s.toLowerCase(Locale.ENGLISH);
        privateKeyEntry.protectedPrivKey = (byte[])encoded.clone();
        if (array != null) {
            if (array.length > 1 && !this.validateChain(array)) {
                throw new KeyStoreException("Certificate chain is not valid");
            }
            privateKeyEntry.chain = (Certificate[])array.clone();
            this.certificateCount += array.length;
            if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("Setting a " + privateKeyEntry.chain.length + "-certificate chain at alias '" + s + "'");
            }
        }
        ++this.privateKeyCount;
        this.entries.put(s.toLowerCase(Locale.ENGLISH), privateKeyEntry);
    }
    
    private byte[] getSalt() {
        byte[] bytes = new byte[20];
        if (this.random == null) {
            this.random = new SecureRandom();
        }
        this.random.nextBytes(bytes);
        return bytes;
    }
    
    private AlgorithmParameters getPBEAlgorithmParameters(String algorithm) throws IOException {
        PBEParameterSpec paramSpec = new PBEParameterSpec(this.getSalt(), 50000);
        AlgorithmParameters instance;
        try {
            instance = AlgorithmParameters.getInstance(algorithm);
            instance.init(paramSpec);
        }
        catch (Exception cause) {
            throw new IOException("getPBEAlgorithmParameters failed: " + cause.getMessage(), cause);
        }
        return instance;
    }
    
    private AlgorithmParameters parseAlgParameters(ObjectIdentifier objectIdentifier, DerInputStream derInputStream) throws IOException {
        AlgorithmParameters algorithmParameters = null;
        try {
            DerValue derValue;
            if (derInputStream.available() == 0) {
                derValue = null;
            }
            else {
                derValue = derInputStream.getDerValue();
                if (derValue.tag == 5) {
                    derValue = null;
                }
            }
            if (derValue != null) {
                if (objectIdentifier.equals(PKCS12KeyStore.pbes2_OID)) {
                    algorithmParameters = AlgorithmParameters.getInstance("PBES2");
                }
                else {
                    algorithmParameters = AlgorithmParameters.getInstance("PBE");
                }
                algorithmParameters.init(derValue.toByteArray());
            }
        }
        catch (Exception cause) {
            throw new IOException("parseAlgParameters failed: " + cause.getMessage(), cause);
        }
        return algorithmParameters;
    }
    
    private SecretKey getPBEKey(char[] password) throws IOException {
        SecretKey generateSecret;
        try {
            PBEKeySpec keySpec = new PBEKeySpec(password);
            generateSecret = SecretKeyFactory.getInstance("PBE").generateSecret(keySpec);
            keySpec.clearPassword();
        }
        catch (Exception cause) {
            throw new IOException("getSecretKey failed: " + cause.getMessage(), cause);
        }
        return generateSecret;
    }
    
    private byte[] encryptPrivateKey(byte[] input, KeyStore.PasswordProtection passwordProtection) throws IOException, NoSuchAlgorithmException, UnrecoverableKeyException {
        byte[] encoded;
        try {
            String protectionAlgorithm = passwordProtection.getProtectionAlgorithm();
            AlgorithmParameters params;
            if (protectionAlgorithm != null) {
                AlgorithmParameterSpec protectionParameters = passwordProtection.getProtectionParameters();
                if (protectionParameters != null) {
                    params = AlgorithmParameters.getInstance(protectionAlgorithm);
                    params.init(protectionParameters);
                }
                else {
                    params = this.getPBEAlgorithmParameters(protectionAlgorithm);
                }
            }
            else {
                protectionAlgorithm = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                    @Override
                    public String run() {
                        String s = Security.getProperty(PKCS12KeyStore.KEY_PROTECTION_ALGORITHM[0]);
                        if (s == null) {
                            s = Security.getProperty(PKCS12KeyStore.KEY_PROTECTION_ALGORITHM[1]);
                            return s;
                        }
                        return s;
                    }
                });
                if (protectionAlgorithm == null || protectionAlgorithm.isEmpty()) {
                    protectionAlgorithm = "PBEWithSHA1AndDESede";
                }
                params = this.getPBEAlgorithmParameters(protectionAlgorithm);
            }
            ObjectIdentifier mapPBEAlgorithmToOID = mapPBEAlgorithmToOID(protectionAlgorithm);
            if (mapPBEAlgorithmToOID == null) {
                throw new IOException("PBE algorithm '" + protectionAlgorithm + " 'is not supported for key entry protection");
            }
            SecretKey pbeKey = this.getPBEKey(passwordProtection.getPassword());
            Cipher instance = Cipher.getInstance(protectionAlgorithm);
            instance.init(1, pbeKey, params);
            byte[] doFinal = instance.doFinal(input);
            AlgorithmId algid = new AlgorithmId(mapPBEAlgorithmToOID, instance.getParameters());
            if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("  (Cipher algorithm: " + instance.getAlgorithm() + ")");
            }
            encoded = new EncryptedPrivateKeyInfo(algid, doFinal).getEncoded();
        }
        catch (Exception cause) {
            UnrecoverableKeyException o = new UnrecoverableKeyException("Encrypt Private Key failed: " + cause.getMessage());
            o.initCause(cause);
            throw o;
        }
        return encoded;
    }
    
    private static ObjectIdentifier mapPBEAlgorithmToOID(String algname) throws NoSuchAlgorithmException {
        if (algname.toLowerCase(Locale.ENGLISH).startsWith("pbewithhmacsha")) {
            return PKCS12KeyStore.pbes2_OID;
        }
        return AlgorithmId.get(algname).getOID();
    }
    
    private static String mapPBEParamsToAlgorithm(ObjectIdentifier objectIdentifier, AlgorithmParameters algorithmParameters) throws NoSuchAlgorithmException {
        if (objectIdentifier.equals(PKCS12KeyStore.pbes2_OID) && algorithmParameters != null) {
            return algorithmParameters.toString();
        }
        return objectIdentifier.toString();
    }
    
    @Override
    public synchronized void engineSetCertificateEntry(String s, Certificate certificate) throws KeyStoreException {
        this.setCertEntry(s, certificate, null);
    }
    
    private void setCertEntry(String str, Certificate certificate, Set<KeyStore.Entry.Attribute> set) throws KeyStoreException {
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(str.toLowerCase(Locale.ENGLISH));
        if (entry != null && entry instanceof PKCS12KeyStore.KeyEntry) {
            throw new KeyStoreException("Cannot overwrite own certificate");
        }
        PKCS12KeyStore.CertEntry certEntry = new PKCS12KeyStore.CertEntry((X509Certificate)certificate, null, str, PKCS12KeyStore.AnyUsage, set);
        ++this.certificateCount;
        this.entries.put(str.toLowerCase(Locale.ENGLISH), certEntry);
        if (PKCS12KeyStore.debug != null) {
            PKCS12KeyStore.debug.println("Setting a trusted certificate at alias '" + str + "'");
        }
        return;
    }
    
    @Override
    public synchronized void engineDeleteEntry(String str) throws KeyStoreException {
        if (PKCS12KeyStore.debug != null) {
            PKCS12KeyStore.debug.println("Removing entry at alias '" + str + "'");
        }
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(str.toLowerCase(Locale.ENGLISH));
        if ((entry instanceof PKCS12KeyStore.PrivateKeyEntry)) {
            PKCS12KeyStore.PrivateKeyEntry privateKeyEntry = (PKCS12KeyStore.PrivateKeyEntry)entry;
            if (privateKeyEntry.chain != null) {
                this.certificateCount -= privateKeyEntry.chain.length;
            }
            --this.privateKeyCount;
        }
        else if ((entry instanceof PKCS12KeyStore.CertEntry)) {
            --this.certificateCount;
        }
        else if ((entry instanceof PKCS12KeyStore.SecretKeyEntry)) {
            --this.secretKeyCount;
        }
        this.entries.remove(str.toLowerCase(Locale.ENGLISH));
    }
    
    @Override
    public Enumeration<String> engineAliases() {
        return Collections.enumeration(this.entries.keySet());
    }
    
    @Override
    public boolean engineContainsAlias(String s) {
        return this.entries.containsKey(s.toLowerCase(Locale.ENGLISH));
    }
    
    @Override
    public int engineSize() {
        return this.entries.size();
    }
    
    @Override
    public boolean engineIsKeyEntry(String s) {
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(s.toLowerCase(Locale.ENGLISH));
        return entry != null && entry instanceof PKCS12KeyStore.KeyEntry;
    }
    
    @Override
    public boolean engineIsCertificateEntry(String s) {
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(s.toLowerCase(Locale.ENGLISH));
        return entry != null && entry instanceof PKCS12KeyStore.CertEntry && ((PKCS12KeyStore.CertEntry)entry).trustedKeyUsage != null;
    }
    
    @Override
    public boolean engineEntryInstanceOf(String s, Class<? extends KeyStore.Entry> clazz) {
        if (clazz == KeyStore.TrustedCertificateEntry.class) {
            return this.engineIsCertificateEntry(s);
        }
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(s.toLowerCase(Locale.ENGLISH));
        if (clazz == KeyStore.PrivateKeyEntry.class) {
            return entry != null && entry instanceof PKCS12KeyStore.PrivateKeyEntry;
        }
        return clazz == KeyStore.SecretKeyEntry.class && entry != null && entry instanceof PKCS12KeyStore.SecretKeyEntry;
    }
    
    @Override
    public String engineGetCertificateAlias(Certificate other) {
        Certificate cert = null;
        Enumeration<String> engineAliases = this.engineAliases();
        while (engineAliases.hasMoreElements()) {
            String s = (String)engineAliases.nextElement();
            PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(s);
            if ((entry instanceof PKCS12KeyStore.PrivateKeyEntry)) {
                if (((PKCS12KeyStore.PrivateKeyEntry)entry).chain != null) {
                    cert = ((PKCS12KeyStore.PrivateKeyEntry)entry).chain[0];
                }
            }
            else {
                if (!(entry instanceof PKCS12KeyStore.CertEntry) || ((PKCS12KeyStore.CertEntry)entry).trustedKeyUsage == null) {
                    continue;
                }
                cert = ((PKCS12KeyStore.CertEntry)entry).cert;
            }
            if (cert != null && cert.equals(other)) {
                return s;
            }
            continue;
        }
        return null;
    }
    
    @Override
    public synchronized void engineStore(OutputStream outputStream, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        if (array == null) {
            throw new IllegalArgumentException("password can't be null");
        }
        DerOutputStream derOutputStream = new DerOutputStream();
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.putInteger(3);
        derOutputStream.write(derOutputStream2.toByteArray());
        DerOutputStream out = new DerOutputStream();
        DerOutputStream derOutputStream3 = new DerOutputStream();
        if (this.privateKeyCount > 0 || this.secretKeyCount > 0) {
            if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("Storing " + (this.privateKeyCount + this.secretKeyCount) + " protected key(s) in a PKCS#7 data");
            }
            new ContentInfo(this.createSafeContent()).encode(derOutputStream3);
        }
        if (this.certificateCount > 0) {
            if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("Storing " + this.certificateCount + " certificate(s) in a PKCS#7 encryptedData");
            }
            new ContentInfo(ContentInfo.ENCRYPTED_DATA_OID, new DerValue(this.createEncryptedData(array))).encode(derOutputStream3);
        }
        DerOutputStream o = new DerOutputStream();
        o.write((byte)48, derOutputStream3);
        byte[] byteArray = o.toByteArray();
        new ContentInfo(byteArray).encode(out);
        derOutputStream.write(out.toByteArray());
        derOutputStream.write(this.calculateMac(array, byteArray));
        DerOutputStream o2 = new DerOutputStream();
        o2.write((byte)48, derOutputStream);
        outputStream.write(o2.toByteArray());
        outputStream.flush();
        return;
    }
    
    @Override
    public KeyStore.Entry engineGetEntry(String str, KeyStore.ProtectionParameter protectionParameter) throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableEntryException {
        if (!this.engineContainsAlias(str)) {
            return null;
        }
        PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(str.toLowerCase(Locale.ENGLISH));
        if (protectionParameter == null) {
            if (this.engineIsCertificateEntry(str)) {
                if (entry instanceof PKCS12KeyStore.CertEntry && ((PKCS12KeyStore.CertEntry)entry).trustedKeyUsage != null) {
                    if (PKCS12KeyStore.debug != null) {
                        PKCS12KeyStore.debug.println("Retrieved a trusted certificate at alias '" + str + "'");
                    }
                    return new KeyStore.TrustedCertificateEntry(((PKCS12KeyStore.CertEntry)entry).cert, this.getAttributes(entry));
                }
            }
            else {
                throw new UnrecoverableKeyException("requested entry requires a password");
            }
        }
        if ((protectionParameter instanceof KeyStore.PasswordProtection)) {
            if (this.engineIsCertificateEntry(str)) {
                throw new UnsupportedOperationException("trusted certificate entries are not password-protected");
            }
            if (this.engineIsKeyEntry(str)) {
                Key engineGetKey = this.engineGetKey(str, ((KeyStore.PasswordProtection)protectionParameter).getPassword());
                if ((engineGetKey instanceof PrivateKey)) {
                    return new KeyStore.PrivateKeyEntry((PrivateKey)engineGetKey, this.engineGetCertificateChain(str), this.getAttributes(entry));
                }
                if ((engineGetKey instanceof SecretKey)) {
                    return new KeyStore.SecretKeyEntry((SecretKey)engineGetKey, this.getAttributes(entry));
                }
            }
            else if (!this.engineIsKeyEntry(str)) {
                throw new UnsupportedOperationException("untrusted certificate entries are not password-protected");
            }
        }
        throw new UnsupportedOperationException();
    }
    
    @Override
    public synchronized void engineSetEntry(String s, KeyStore.Entry entry, KeyStore.ProtectionParameter protectionParameter) throws KeyStoreException {
        if (protectionParameter != null && !(protectionParameter instanceof KeyStore.PasswordProtection)) {
            throw new KeyStoreException("unsupported protection parameter");
        }
        KeyStore.PasswordProtection passwordProtection = null;
        if (protectionParameter != null) {
            passwordProtection = (KeyStore.PasswordProtection)protectionParameter;
        }
        if ((entry instanceof KeyStore.TrustedCertificateEntry)) {
            if (protectionParameter != null && passwordProtection.getPassword() != null) {
                throw new KeyStoreException("trusted certificate entries are not password-protected");
            }
            KeyStore.TrustedCertificateEntry trustedCertificateEntry = (KeyStore.TrustedCertificateEntry)entry;
            this.setCertEntry(s, trustedCertificateEntry.getTrustedCertificate(), trustedCertificateEntry.getAttributes());
            return;
        }
        else if ((entry instanceof KeyStore.PrivateKeyEntry)) {
            if (passwordProtection == null || passwordProtection.getPassword() == null) {
                throw new KeyStoreException("non-null password required to create PrivateKeyEntry");
            }
            KeyStore.PrivateKeyEntry privateKeyEntry = (KeyStore.PrivateKeyEntry)entry;
            this.setKeyEntry(s, privateKeyEntry.getPrivateKey(), passwordProtection, privateKeyEntry.getCertificateChain(), privateKeyEntry.getAttributes());
            return;
        }
        else if ((entry instanceof KeyStore.SecretKeyEntry)) {
            if (passwordProtection == null || passwordProtection.getPassword() == null) {
                throw new KeyStoreException("non-null password required to create SecretKeyEntry");
            }
            KeyStore.SecretKeyEntry secretKeyEntry = (KeyStore.SecretKeyEntry)entry;
            this.setKeyEntry(s, secretKeyEntry.getSecretKey(), passwordProtection, (Certificate[])null, secretKeyEntry.getAttributes());
            return;
        }
        else {
            throw new KeyStoreException("unsupported entry type: " + entry.getClass().getName());
        }
    }
    
    private Set<KeyStore.Entry.Attribute> getAttributes(PKCS12KeyStore.Entry entry) {
        if (entry.attributes == null) {
            entry.attributes = new HashSet();
        }
        entry.attributes.add(new PKCS12Attribute(PKCS12KeyStore.PKCS9FriendlyName_OID.toString(), entry.alias));
        byte[] keyId = entry.keyId;
        if (keyId != null) {
            entry.attributes.add(new PKCS12Attribute(PKCS12KeyStore.PKCS9LocalKeyId_OID.toString(), Debug.toString(keyId)));
        }
        if ((entry instanceof PKCS12KeyStore.CertEntry)) {
            ObjectIdentifier[] trustedKeyUsage = ((PKCS12KeyStore.CertEntry)entry).trustedKeyUsage;
            if (trustedKeyUsage != null) {
                if (trustedKeyUsage.length == 1) {
                    entry.attributes.add(new PKCS12Attribute(PKCS12KeyStore.TrustedKeyUsage_OID.toString(), trustedKeyUsage[0].toString()));
                }
                else {
                    entry.attributes.add(new PKCS12Attribute(PKCS12KeyStore.TrustedKeyUsage_OID.toString(), Arrays.toString(trustedKeyUsage)));
                }
            }
        }
        return entry.attributes;
    }
    
    private byte[] generateHash(byte[] input) throws IOException {
        byte[] digest;
        try {
            MessageDigest instance = MessageDigest.getInstance("SHA1");
            instance.update(input);
            digest = instance.digest();
        }
        catch (Exception ex) {
            throw new IOException("generateHash failed: " + ex, ex);
        }
        return digest;
    }
    
    private byte[] calculateMac(char[] array, byte[] input) throws IOException {
        String s = "SHA1";
        byte[] byteArray;
        try {
            byte[] salt = this.getSalt();
            Mac instance = Mac.getInstance("HmacPBESHA1");
            instance.init(this.getPBEKey(array), new PBEParameterSpec(salt, 100000));
            instance.update(input);
            MacData macData = new MacData(s, instance.doFinal(), salt, 100000);
            DerOutputStream derOutputStream = new DerOutputStream();
            derOutputStream.write(macData.getEncoded());
            byteArray = derOutputStream.toByteArray();
        }
        catch (Exception ex) {
            throw new IOException("calculateMac failed: " + ex, ex);
        }
        return byteArray;
    }
    
    private boolean validateChain(Certificate[] a) {
        int i = 0;
        while (i < a.length - 1) {
            if (!((X509Certificate)a[i]).getIssuerX500Principal().equals(((X509Certificate)a[i + 1]).getSubjectX500Principal())) {
                return false;
            }
            ++i;
        }
        return new HashSet(Arrays.asList(a)).size() == a.length;
    }
    
    private byte[] getBagAttributes(String s, byte[] array, Set<KeyStore.Entry.Attribute> set) throws IOException {
        return this.getBagAttributes(s, array, null, set);
    }
    
    private byte[] getBagAttributes(String s, byte[] array, ObjectIdentifier[] array2, Set<KeyStore.Entry.Attribute> set) throws IOException {
        byte[] byteArray = null;
        byte[] byteArray2 = null;
        byte[] byteArray3 = null;
        if (s == null && array == null && byteArray3 == null) {
            return null;
        }
        DerOutputStream derOutputStream = new DerOutputStream();
        if (s != null) {
            DerOutputStream derOutputStream2 = new DerOutputStream();
            derOutputStream2.putOID(PKCS12KeyStore.PKCS9FriendlyName_OID);
            DerOutputStream derOutputStream3 = new DerOutputStream();
            DerOutputStream derOutputStream4 = new DerOutputStream();
            derOutputStream3.putBMPString(s);
            derOutputStream2.write((byte)49, derOutputStream3);
            derOutputStream4.write((byte)48, derOutputStream2);
            byteArray2 = derOutputStream4.toByteArray();
        }
        if (array != null) {
            DerOutputStream o = new DerOutputStream();
            o.putOID(PKCS12KeyStore.PKCS9LocalKeyId_OID);
            DerOutputStream o2 = new DerOutputStream();
            DerOutputStream o3 = new DerOutputStream();
            o2.putOctetString(array);
            o.write((byte)49, o2);
            o3.write((byte)48, o);
            byteArray = o3.toByteArray();
        }
        if (array2 != null) {
            DerOutputStream o4 = new DerOutputStream();
            o4.putOID(PKCS12KeyStore.TrustedKeyUsage_OID);
            DerOutputStream o5 = new DerOutputStream();
            DerOutputStream o6 = new DerOutputStream();
            for (int length = array2.length, i = 0; i < length; ++i) {
                o5.putOID(array2[i]);
            }
            o4.write((byte)49, o5);
            o6.write((byte)48, o4);
            byteArray3 = o6.toByteArray();
        }
        DerOutputStream o7 = new DerOutputStream();
        if (byteArray2 != null) {
            o7.write(byteArray2);
        }
        if (byteArray != null) {
            o7.write(byteArray);
        }
        if (byteArray3 != null) {
            o7.write(byteArray3);
        }
        if (set != null) {
            for (KeyStore.Entry.Attribute attribute : set) {
                String name = attribute.getName();
                if (PKCS12KeyStore.CORE_ATTRIBUTES[0].equals(name) || PKCS12KeyStore.CORE_ATTRIBUTES[1].equals(name)) {
                    continue;
                }
                if (PKCS12KeyStore.CORE_ATTRIBUTES[2].equals(name)) {
                    continue;
                }
                o7.write(((PKCS12Attribute)attribute).getEncoded());
            }
        }
        derOutputStream.write((byte)49, o7);
        return derOutputStream.toByteArray();
    }
    
    private byte[] createEncryptedData(char[] array) throws CertificateException, IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        Enumeration<String> engineAliases = this.engineAliases();
        while (engineAliases.hasMoreElements()) {
            PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get((Object)engineAliases.nextElement());
            Certificate[] chain;
            if ((entry instanceof PKCS12KeyStore.PrivateKeyEntry)) {
                PKCS12KeyStore.PrivateKeyEntry privateKeyEntry = (PKCS12KeyStore.PrivateKeyEntry)entry;
                if (privateKeyEntry.chain != null) {
                    chain = privateKeyEntry.chain;
                }
                else {
                    chain = new Certificate[0];
                }
            }
            else if ((entry instanceof PKCS12KeyStore.CertEntry)) {
                chain = new Certificate[] { ((PKCS12KeyStore.CertEntry)entry).cert };
            }
            else {
                chain = new Certificate[0];
            }
            for (int i = 0; i < chain.length; ++i) {
                DerOutputStream o = new DerOutputStream();
                o.putOID(PKCS12KeyStore.CertBag_OID);
                DerOutputStream o2 = new DerOutputStream();
                o2.putOID(PKCS12KeyStore.PKCS9CertType_OID);
                DerOutputStream o3 = new DerOutputStream();
                X509Certificate x509Certificate = (X509Certificate)chain[i];
                o3.putOctetString(x509Certificate.getEncoded());
                o2.write(DerValue.createTag((byte)(-128), true, (byte)0), o3);
                DerOutputStream o4 = new DerOutputStream();
                o4.write((byte)48, o2);
                byte[] byteArray = o4.toByteArray();
                DerOutputStream o5 = new DerOutputStream();
                o5.write(byteArray);
                o.write(DerValue.createTag((byte)(-128), true, (byte)0), o5);
                byte[] b;
                if (i == 0) {
                    if ((entry instanceof PKCS12KeyStore.KeyEntry)) {
                        PKCS12KeyStore.KeyEntry keyEntry = (PKCS12KeyStore.KeyEntry)entry;
                        b = this.getBagAttributes(keyEntry.alias, keyEntry.keyId, keyEntry.attributes);
                    }
                    else {
                        PKCS12KeyStore.CertEntry certEntry = (PKCS12KeyStore.CertEntry)entry;
                        b = this.getBagAttributes(certEntry.alias, certEntry.keyId, certEntry.trustedKeyUsage, certEntry.attributes);
                    }
                }
                else {
                    b = this.getBagAttributes(x509Certificate.getSubjectX500Principal().getName(), null, entry.attributes);
                }
                if (b != null) {
                    o.write(b);
                }
                derOutputStream.write((byte)48, o);
            }
        }
        DerOutputStream o6 = new DerOutputStream();
        o6.write((byte)48, derOutputStream);
        byte[] encryptContent = this.encryptContent(o6.toByteArray(), array);
        DerOutputStream o7 = new DerOutputStream();
        DerOutputStream o8 = new DerOutputStream();
        o7.putInteger(0);
        o7.write(encryptContent);
        o8.write((byte)48, o7);
        return o8.toByteArray();
    }
    
    private byte[] createSafeContent() throws CertificateException, IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        Enumeration<String> engineAliases = this.engineAliases();
        while (engineAliases.hasMoreElements()) {
            String s = (String)engineAliases.nextElement();
            PKCS12KeyStore.Entry entry = (PKCS12KeyStore.Entry)this.entries.get(s);
            if (entry == null) {
                continue;
            }
            if (!(entry instanceof PKCS12KeyStore.KeyEntry)) {
                continue;
            }
            DerOutputStream o = new DerOutputStream();
            PKCS12KeyStore.KeyEntry keyEntry = (PKCS12KeyStore.KeyEntry)entry;
            if ((keyEntry instanceof PKCS12KeyStore.PrivateKeyEntry)) {
                o.putOID(PKCS12KeyStore.PKCS8ShroudedKeyBag_OID);
                byte[] protectedPrivKey = ((PKCS12KeyStore.PrivateKeyEntry)keyEntry).protectedPrivKey;
                Object o6;
                try {
                    o6 = new EncryptedPrivateKeyInfo(protectedPrivKey);
                }
                catch (IOException ex) {
                    throw new IOException("Private key not stored as PKCS#8 EncryptedPrivateKeyInfo" + ex.getMessage());
                }
                DerOutputStream o7 = new DerOutputStream();
                o7.write(o6.getEncoded());
                o.write(DerValue.createTag((byte)(-128), true, (byte)0), o7);
            }
            else {
                if (!(keyEntry instanceof PKCS12KeyStore.SecretKeyEntry)) {
                    continue;
                }
                o.putOID(PKCS12KeyStore.SecretBag_OID);
                DerOutputStream o2 = new DerOutputStream();
                o2.putOID(PKCS12KeyStore.PKCS8ShroudedKeyBag_OID);
                DerOutputStream o3 = new DerOutputStream();
                o3.putOctetString(((PKCS12KeyStore.SecretKeyEntry)keyEntry).protectedSecretKey);
                o2.write(DerValue.createTag((byte)(-128), true, (byte)0), o3);
                DerOutputStream o4 = new DerOutputStream();
                o4.write((byte)48, o2);
                byte[] byteArray = o4.toByteArray();
                DerOutputStream o5 = new DerOutputStream();
                o5.write(byteArray);
                o.write(DerValue.createTag((byte)(-128), true, (byte)0), o5);
            }
            o.write(this.getBagAttributes(s, entry.keyId, entry.attributes));
            derOutputStream.write((byte)48, o);
        }
        DerOutputStream o8 = new DerOutputStream();
        o8.write((byte)48, derOutputStream);
        return o8.toByteArray();
    }
    
    private byte[] encryptContent(byte[] input, char[] array) throws IOException {
        AlgorithmParameters pbeAlgorithmParameters = this.getPBEAlgorithmParameters("PBEWithSHA1AndRC2_40");
        DerOutputStream out = new DerOutputStream();
        new AlgorithmId(PKCS12KeyStore.pbeWithSHAAnd40BitRC2CBC_OID, pbeAlgorithmParameters).encode(out);
        byte[] byteArray = out.toByteArray();
        byte[] doFinal;
        try {
            SecretKey pbeKey = this.getPBEKey(array);
            Cipher instance = Cipher.getInstance("PBEWithSHA1AndRC2_40");
            instance.init(1, pbeKey, pbeAlgorithmParameters);
            doFinal = instance.doFinal(input);
            if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("  (Cipher algorithm: " + instance.getAlgorithm() + ")");
            }
        }
        catch (Exception ex) {
            throw new IOException("Failed to encrypt safe contents entry: " + ex, ex);
        }
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.putOID(ContentInfo.DATA_OID);
        derOutputStream.write(byteArray);
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.putOctetString(doFinal);
        derOutputStream.writeImplicit(DerValue.createTag((byte)(-128), false, (byte)0), derOutputStream2);
        DerOutputStream derOutputStream3 = new DerOutputStream();
        derOutputStream3.write((byte)48, derOutputStream);
        return derOutputStream3.toByteArray();
    }
    
    @Override
    public synchronized void engineLoad(InputStream in, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        if (in == null) {
            return;
        }
        this.counter = 0;
        DerInputStream derInputStream = new DerValue(in).toDerInputStream();
        if (derInputStream.getInteger() != 3) {
            throw new IOException("PKCS12 keystore not in version 3 format");
        }
        this.entries.clear();
        ContentInfo contentInfo = new ContentInfo(derInputStream);
        if (contentInfo.getContentType().equals(ContentInfo.DATA_OID)) {
            byte[] data = contentInfo.getData();
            DerValue[] sequence = new DerInputStream(data).getSequence(2);
            int length = sequence.length;
            this.privateKeyCount = 0;
            this.secretKeyCount = 0;
            this.certificateCount = 0;
            for (int i = 0; i < length; ++i) {
                ContentInfo o = new ContentInfo(new DerInputStream(sequence[i].toByteArray()));
                ObjectIdentifier contentType = o.getContentType();
                byte[] array2;
                if (contentType.equals(ContentInfo.DATA_OID)) {
                    if (PKCS12KeyStore.debug != null) {
                        PKCS12KeyStore.debug.println("Loading PKCS#7 data");
                    }
                    array2 = o.getData();
                }
                else if (contentType.equals(ContentInfo.ENCRYPTED_DATA_OID)) {
                    if (array == null) {
                        if (PKCS12KeyStore.debug != null) {
                            PKCS12KeyStore.debug.println("Warning: skipping PKCS#7 encryptedData - no password was supplied");
                            continue;
                        }
                        continue;
                    }
                    else {
                        DerInputStream derInputStream2 = o.getContent().toDerInputStream();
                        derInputStream2.getInteger();
                        DerValue[] sequence2 = derInputStream2.getSequence(3);
                        if (sequence2.length != 3) {
                            throw new IOException("Invalid length for EncryptedContentInfo");
                        }
                        sequence2[0].getOID();
                        sequence2[1].toByteArray();
                        if (!sequence2[2].isContextSpecific((byte)0)) {
                            throw new IOException("unsupported encrypted content type " + (int)sequence2[2].tag);
                        }
                        int tag = 4;
                        if (sequence2[2].isConstructed()) {
                            tag = (byte)(tag | 0x20);
                        }
                        sequence2[2].resetTag((byte)tag);
                        array2 = sequence2[2].getOctetString();
                        DerInputStream derInputStream3 = sequence2[1].toDerInputStream();
                        ObjectIdentifier oid = derInputStream3.getOID();
                        AlgorithmParameters algParameters = this.parseAlgParameters(oid, derInputStream3);
                        int iterationCount = 0;
                        if (algParameters != null) {
                            PBEParameterSpec pbeParameterSpec;
                            try {
                                pbeParameterSpec = (PBEParameterSpec)algParameters.getParameterSpec(PBEParameterSpec.class);
                            }
                            catch (InvalidParameterSpecException ex) {
                                throw new IOException("Invalid PBE algorithm parameters");
                            }
                            iterationCount = pbeParameterSpec.getIterationCount();
                            if (iterationCount > 5000000) {
                                throw new IOException("PBE iteration count too large");
                            }
                        }
                        if (PKCS12KeyStore.debug != null) {
                            PKCS12KeyStore.debug.println("Loading PKCS#7 encryptedData (" + new AlgorithmId(oid).getName() + " iterations: " + iterationCount + ")");
                        }
                        Label_0566: {
                            try {
                                SecretKey pbeKey = this.getPBEKey(array);
                                Cipher instance = Cipher.getInstance(oid.toString());
                                instance.init(2, pbeKey, algParameters);
                                array2 = instance.doFinal(array2);
                            }
                            catch (Exception obj) {
                                if (array.length == 0) {
                                    array = new char[] { '\0' };
                                    continue;
                                }
                                throw new IOException("keystore password was incorrect", new UnrecoverableKeyException("failed to decrypt safe contents entry: " + obj));
                            }
                        }
                    }
                }
                else {
                    throw new IOException("public key protected PKCS12 not supported");
                }
                this.loadSafeContents(new DerInputStream(array2), array);
            }
            if (array != null && derInputStream.available() > 0) {
                MacData o2 = new MacData(derInputStream);
                int iterations = o2.getIterations();
                try {
                    if (iterations > 5000000) {
                        throw new InvalidAlgorithmParameterException("MAC iteration count too large: " + iterations);
                    }
                    Mac instance2 = Mac.getInstance("HmacPBE" + o2.getDigestAlgName().toUpperCase(Locale.ENGLISH).replace("-", ""));
                    instance2.init(this.getPBEKey(array), new PBEParameterSpec(o2.getSalt(), iterations));
                    instance2.update(data);
                    byte[] doFinal = instance2.doFinal();
                    if (PKCS12KeyStore.debug != null) {
                        PKCS12KeyStore.debug.println("Checking keystore integrity (" + instance2.getAlgorithm() + " iterations: " + iterations + ")");
                    }
                    if (!MessageDigest.isEqual(o2.getDigest(), doFinal)) {
                        throw new UnrecoverableKeyException("Failed PKCS12 integrity checking");
                    }
                }
                catch (Exception ex2) {
                    throw new IOException("Integrity check failed: " + ex2, ex2);
                }
            }
            PKCS12KeyStore.PrivateKeyEntry[] array3 = (PKCS12KeyStore.PrivateKeyEntry[])this.keyList.toArray(new PKCS12KeyStore.PrivateKeyEntry[this.keyList.size()]);
            for (int j = 0; j < array3.length; ++j) {
                PKCS12KeyStore.PrivateKeyEntry privateKeyEntry = array3[j];
                if (privateKeyEntry.keyId != null) {
                    ArrayList o3 = new ArrayList();
                    X509Certificate matchedCertificate = this.findMatchedCertificate(privateKeyEntry);
                Label_1181:
                    while (matchedCertificate != null) {
                        if (!o3.isEmpty()) {
                            Iterator<Object> iterator = o3.iterator();
                            while (iterator.hasNext()) {
                                if (matchedCertificate.equals((Object)iterator.next())) {
                                    if (PKCS12KeyStore.debug != null) {
                                        PKCS12KeyStore.debug.println("Loop detected in certificate chain. Skip adding repeated cert to chain. Subject: " + matchedCertificate.getSubjectX500Principal().toString());
                                        break Label_1181;
                                    }
                                    break Label_1181;
                                }
                                else {
                                    continue;
                                }
                            }
                        }
                        o3.add(matchedCertificate);
                        X500Principal issuerX500Principal = matchedCertificate.getIssuerX500Principal();
                        if (issuerX500Principal.equals(matchedCertificate.getSubjectX500Principal())) {
                            break;
                        }
                        matchedCertificate = (X509Certificate)this.certsMap.get(issuerX500Principal);
                    }
                    if (o3.size() > 0) {
                        privateKeyEntry.chain = (Certificate[])o3.toArray(new Certificate[o3.size()]);
                    }
                }
            }
            if (PKCS12KeyStore.debug != null) {
                if (this.privateKeyCount > 0) {
                    PKCS12KeyStore.debug.println("Loaded " + this.privateKeyCount + " protected private key(s)");
                }
                if (this.secretKeyCount > 0) {
                    PKCS12KeyStore.debug.println("Loaded " + this.secretKeyCount + " protected secret key(s)");
                }
                if (this.certificateCount > 0) {
                    PKCS12KeyStore.debug.println("Loaded " + this.certificateCount + " certificate(s)");
                }
            }
            this.certEntries.clear();
            this.certsMap.clear();
            this.keyList.clear();
            return;
        }
        else {
            throw new IOException("public key protected PKCS12 not supported");
        }
    }
    
    private X509Certificate findMatchedCertificate(PKCS12KeyStore.PrivateKeyEntry privateKeyEntry) {
        PKCS12KeyStore.CertEntry certEntry = null;
        PKCS12KeyStore.CertEntry certEntry2 = null;
        for (PKCS12KeyStore.CertEntry certEntry3 : this.certEntries) {
            if (Arrays.equals(privateKeyEntry.keyId, certEntry3.keyId)) {
                certEntry = certEntry3;
                if (privateKeyEntry.alias.equalsIgnoreCase(certEntry3.alias)) {
                    return certEntry3.cert;
                }
                continue;
            }
            else if (privateKeyEntry.alias.equalsIgnoreCase(certEntry3.alias)) {
                certEntry2 = certEntry3;
            }
            else {
                continue;
            }
        }
        if (certEntry != null) {
            return certEntry.cert;
        }
        if (certEntry2 != null) {
            return certEntry2.cert;
        }
        return null;
    }
    
    private void loadSafeContents(DerInputStream derInputStream, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        DerValue[] sequence = derInputStream.getSequence(2);
        int length = sequence.length;
        int i = 0;
        while (i < length) {
            Object o = null;
            DerInputStream derInputStream2 = sequence[i].toDerInputStream();
            ObjectIdentifier oid = derInputStream2.getOID();
            DerValue derValue = derInputStream2.getDerValue();
            if (!derValue.isContextSpecific((byte)0)) {
                throw new IOException("unsupported PKCS12 bag value type " + (int)derValue.tag);
            }
            DerValue derValue2 = derValue.data.getDerValue();
            if (oid.equals(PKCS12KeyStore.PKCS8ShroudedKeyBag_OID)) {
                PKCS12KeyStore.PrivateKeyEntry o3 = new PKCS12KeyStore.PrivateKeyEntry();
                o3.protectedPrivKey = derValue2.toByteArray();
                o = o3;
                ++this.privateKeyCount;
            }
            else if (oid.equals(PKCS12KeyStore.CertBag_OID)) {
                DerValue[] sequence3 = new DerInputStream(derValue2.toByteArray()).getSequence(2);
                if (sequence3.length != 2) {
                    throw new IOException("Invalid length for CertBag");
                }
                sequence3[0].getOID();
                if (!sequence3[1].isContextSpecific((byte)0)) {
                    throw new IOException("unsupported PKCS12 cert value type " + (int)sequence3[1].tag);
                }
                o = (Certificate)CertificateFactory.getInstance("X509").generateCertificate(new ByteArrayInputStream(sequence3[1].data.getDerValue().getOctetString()));
                ++this.certificateCount;
            }
            else if (oid.equals(PKCS12KeyStore.SecretBag_OID)) {
                DerValue[] sequence2 = new DerInputStream(derValue2.toByteArray()).getSequence(2);
                if (sequence2.length != 2) {
                    throw new IOException("Invalid length for SecretBag");
                }
                sequence2[0].getOID();
                if (!sequence2[1].isContextSpecific((byte)0)) {
                    throw new IOException("unsupported PKCS12 secret value type " + (int)sequence2[1].tag);
                }
                DerValue derValue3 = sequence2[1].data.getDerValue();
                PKCS12KeyStore.SecretKeyEntry o2 = new PKCS12KeyStore.SecretKeyEntry();
                o2.protectedSecretKey = derValue3.getOctetString();
                o = o2;
                ++this.secretKeyCount;
            }
            else if (PKCS12KeyStore.debug != null) {
                PKCS12KeyStore.debug.println("Unsupported PKCS12 bag type: " + oid);
            }
            DerValue[] set;
            try {
                set = derInputStream2.getSet(3);
            }
            catch (IOException ex) {
                set = null;
            }
            String alias = null;
            byte[] array2 = null;
            ObjectIdentifier[] array3 = null;
            HashSet o4 = new HashSet();
            if (set != null) {
                int j = 0;
                while (j < set.length) {
                    byte[] byteArray = set[j].toByteArray();
                    DerValue[] sequence4 = new DerInputStream(byteArray).getSequence(2);
                    if (sequence4.length != 2) {
                        throw new IOException("Invalid length for Attribute");
                    }
                    ObjectIdentifier oid2 = sequence4[0].getOID();
                    DerInputStream o5 = new DerInputStream(sequence4[1].toByteArray());
                    DerValue[] set2;
                    try {
                        set2 = o5.getSet(1);
                    }
                    catch (IOException ex2) {
                        throw new IOException("Attribute " + oid2 + " should have a value " + ex2.getMessage());
                    }
                    if (oid2.equals(PKCS12KeyStore.PKCS9FriendlyName_OID)) {
                        alias = set2[0].getBMPString();
                    }
                    else if (oid2.equals(PKCS12KeyStore.PKCS9LocalKeyId_OID)) {
                        array2 = set2[0].getOctetString();
                    }
                    else if (oid2.equals(PKCS12KeyStore.TrustedKeyUsage_OID)) {
                        array3 = new ObjectIdentifier[set2.length];
                        for (int k = 0; k < set2.length; ++k) {
                            array3[k] = set2[k].getOID();
                        }
                    }
                    else {
                        o4.add(new PKCS12Attribute(byteArray));
                    }
                    ++j;
                }
            }
            Label_1174: {
                if ((o instanceof PKCS12KeyStore.KeyEntry)) {
                    PKCS12KeyStore.KeyEntry keyEntry = (PKCS12KeyStore.KeyEntry)o;
                    if (o instanceof PKCS12KeyStore.PrivateKeyEntry && array2 == null) {
                        if (this.privateKeyCount == 1) {
                            array2 = "01".getBytes("UTF8");
                        }
                        else {
                            break Label_1174;
                        }
                    }
                    keyEntry.keyId = array2;
                    String o6 = new String(array2, "UTF8");
                    Object date = null;
                    if (o6.startsWith("Time ")) {
                        try {
                            date = new Date(Long.parseLong(o6.substring(5)));
                        }
                        catch (Exception ex3) {
                            date = null;
                        }
                    }
                    if (date == null) {
                        date = new Date();
                    }
                    keyEntry.date = date;
                    if ((o instanceof PKCS12KeyStore.PrivateKeyEntry)) {
                        this.keyList.add((PKCS12KeyStore.PrivateKeyEntry)keyEntry);
                    }
                    if (keyEntry.attributes == null) {
                        keyEntry.attributes = new HashSet();
                    }
                    keyEntry.attributes.addAll(o4);
                    if (alias == null) {
                        alias = this.getUnfriendlyName();
                    }
                    keyEntry.alias = alias;
                    this.entries.put(alias.toLowerCase(Locale.ENGLISH), keyEntry);
                }
                else if ((o instanceof X509Certificate)) {
                    X509Certificate value = (X509Certificate)o;
                    if (array2 == null && this.privateKeyCount == 1 && i == 0) {
                        array2 = "01".getBytes("UTF8");
                    }
                    if (array3 != null) {
                        if (alias == null) {
                            alias = this.getUnfriendlyName();
                        }
                        this.entries.put(alias.toLowerCase(Locale.ENGLISH), new PKCS12KeyStore.CertEntry(value, array2, alias, array3, o4));
                    }
                    else {
                        this.certEntries.add(new PKCS12KeyStore.CertEntry(value, array2, alias));
                    }
                    X500Principal subjectX500Principal = value.getSubjectX500Principal();
                    if (subjectX500Principal != null && !this.certsMap.containsKey(subjectX500Principal)) {
                        this.certsMap.put(subjectX500Principal, value);
                    }
                }
            }
            ++i;
        }
    }
    
    private String getUnfriendlyName() {
        ++this.counter;
        return String.valueOf(this.counter);
    }
    
    static {
        KEY_PROTECTION_ALGORITHM = new String[] { "keystore.pkcs12.keyProtectionAlgorithm", "keystore.PKCS12.keyProtectionAlgorithm" };
        CORE_ATTRIBUTES = new String[] { "1.2.840.113549.1.9.20", "1.2.840.113549.1.9.21", "2.16.840.1.113894.746875.1.1" };
        debug = Debug.getInstance("pkcs12");
        keyBag = new int[] { 1, 2, 840, 113549, 1, 12, 10, 1, 2 };
        certBag = new int[] { 1, 2, 840, 113549, 1, 12, 10, 1, 3 };
        secretBag = new int[] { 1, 2, 840, 113549, 1, 12, 10, 1, 5 };
        pkcs9Name = new int[] { 1, 2, 840, 113549, 1, 9, 20 };
        pkcs9KeyId = new int[] { 1, 2, 840, 113549, 1, 9, 21 };
        pkcs9certType = new int[] { 1, 2, 840, 113549, 1, 9, 22, 1 };
        pbeWithSHAAnd40BitRC2CBC = new int[] { 1, 2, 840, 113549, 1, 12, 1, 6 };
        pbeWithSHAAnd3KeyTripleDESCBC = new int[] { 1, 2, 840, 113549, 1, 12, 1, 3 };
        pbes2 = new int[] { 1, 2, 840, 113549, 1, 5, 13 };
        TrustedKeyUsage = new int[] { 2, 16, 840, 1, 113894, 746875, 1, 1 };
        AnyExtendedKeyUsage = new int[] { 2, 5, 29, 37, 0 };
        try {
            PKCS12KeyStore.PKCS8ShroudedKeyBag_OID = new ObjectIdentifier(PKCS12KeyStore.keyBag);
            PKCS12KeyStore.CertBag_OID = new ObjectIdentifier(PKCS12KeyStore.certBag);
            PKCS12KeyStore.SecretBag_OID = new ObjectIdentifier(PKCS12KeyStore.secretBag);
            PKCS12KeyStore.PKCS9FriendlyName_OID = new ObjectIdentifier(PKCS12KeyStore.pkcs9Name);
            PKCS12KeyStore.PKCS9LocalKeyId_OID = new ObjectIdentifier(PKCS12KeyStore.pkcs9KeyId);
            PKCS12KeyStore.PKCS9CertType_OID = new ObjectIdentifier(PKCS12KeyStore.pkcs9certType);
            PKCS12KeyStore.pbeWithSHAAnd40BitRC2CBC_OID = new ObjectIdentifier(PKCS12KeyStore.pbeWithSHAAnd40BitRC2CBC);
            PKCS12KeyStore.pbeWithSHAAnd3KeyTripleDESCBC_OID = new ObjectIdentifier(PKCS12KeyStore.pbeWithSHAAnd3KeyTripleDESCBC);
            PKCS12KeyStore.pbes2_OID = new ObjectIdentifier(PKCS12KeyStore.pbes2);
            PKCS12KeyStore.TrustedKeyUsage_OID = new ObjectIdentifier(PKCS12KeyStore.TrustedKeyUsage);
            PKCS12KeyStore.AnyUsage = new ObjectIdentifier[] { new ObjectIdentifier(PKCS12KeyStore.AnyExtendedKeyUsage) };
        }
        catch (IOException ex) {}
    }
    
    private static class Entry
    {
        Date date;
        String alias;
        byte[] keyId;
        Set<KeyStore.Entry.Attribute> attributes;
    }
    
    private static class KeyEntry extends PKCS12KeyStore.Entry
    {
    }
    
    private static class PrivateKeyEntry extends PKCS12KeyStore.KeyEntry
    {
        byte[] protectedPrivKey;
        Certificate[] chain;
    }
    
    private static class SecretKeyEntry extends PKCS12KeyStore.KeyEntry
    {
        byte[] protectedSecretKey;
    }
    
    private static class CertEntry extends PKCS12KeyStore.Entry
    {
        final X509Certificate cert;
        ObjectIdentifier[] trustedKeyUsage;
        
        CertEntry(X509Certificate x509Certificate, byte[] array, String s) {
            this(x509Certificate, array, s, null, null);
        }
        
        CertEntry(X509Certificate cert, byte[] keyId, String alias, ObjectIdentifier[] trustedKeyUsage, Set<? extends KeyStore.Entry.Attribute> set) {
            this.date = new Date();
            this.cert = cert;
            this.keyId = keyId;
            this.alias = alias;
            this.trustedKeyUsage = trustedKeyUsage;
            this.attributes = new HashSet();
            if (set != null) {
                this.attributes.addAll(set);
            }
        }
    }
}
